﻿#include <tbaseutil/tecmahelper.h>
#include <tbaseutil/terror.h>
#include <twidget/ttableviewdialog.h>
#include <QScriptEngine>
#include <QScriptContext>
#include "ttableviewdialog_ecmabinding.h"

Q_DECLARE_METATYPE(QAbstractItemView::SelectionMode)

void TTableViewDialog_EcmaBinding::initBinding(QScriptEngine *iEngine)
{
    iEngine->setDefaultPrototype(qMetaTypeId<TTableViewDialog*>(), QScriptValue());
    QScriptValue proto = iEngine->newVariant(qVariantFromValue((TTableViewDialog*)0));
    proto.setPrototype(iEngine->defaultPrototype(qMetaTypeId<TTableViewDialog*>()));

    TEcmaHelper::registerQObjectMetaType<TTableViewDialog*>(iEngine);
    qScriptRegisterMetaType(iEngine, toScriptValue, fromScriptValue);
    TECMA_BIND_METHOD_FUNCTION(setSelectionMode, 1);

    QScriptValue ctor = iEngine->newFunction(create, proto);

    iEngine->globalObject().setProperty("TTableViewDialog", ctor, QScriptValue::SkipInEnumeration);
}

QScriptValue TTableViewDialog_EcmaBinding::create(QScriptContext *iContext,
                                           QScriptEngine *iEngine)
{
    Q_UNUSED(iEngine)
    if (iContext->thisObject().strictlyEquals(iContext->engine()->globalObject())) {
        return TEcmaHelper::throwError_ForgetNew("TTableViewDialog", iContext);
    }
    if (iContext->argumentCount() == 0) {
        return iContext->engine()->newQObject(iContext->thisObject(),
                                              new TTableViewDialog,
                                              QScriptEngine::AutoOwnership);
    }
    else if(iContext->argumentCount() == 1) {
        QWidget *parent = qobject_cast<QWidget *>(iContext->argument(0).toQObject());
        return iContext->engine()->newQObject(iContext->thisObject(),
                          new TTableViewDialog(parent),
                          QScriptEngine::AutoOwnership);
    }
    else
    {
        return TEcmaHelper::throwError_CtorNotFound("TTableViewDialog", iContext);
    }
}

QScriptValue TTableViewDialog_EcmaBinding::toScriptValue(QScriptEngine *iEngine,
                                                         const QAbstractItemView::SelectionMode &iMode)
{
    return iEngine->newVariant(iMode);
}

void TTableViewDialog_EcmaBinding::fromScriptValue(const QScriptValue &iValue,
                                                  QAbstractItemView::SelectionMode &iMode)
{
    qint32 tmpInt = iValue.toInt32();
    switch(tmpInt){
    case 0: iMode = QAbstractItemView::NoSelection;
        break;
    case 1:
        iMode = QAbstractItemView::SingleSelection;
        break;
    case 2:
        iMode = QAbstractItemView::MultiSelection;
        break;
    case 3:
        iMode = QAbstractItemView::ExtendedSelection;
        break;
    case 4:
        iMode = QAbstractItemView::ContiguousSelection;
        break;
    default:
        iMode = QAbstractItemView::NoSelection;
    }
}

QScriptValue TTableViewDialog_EcmaBinding::setSelectionMode(QScriptContext *iContext,
                                                     QScriptEngine *iEngine)
{
    Q_UNUSED(iEngine)
    TECMA_BIND_CHECK_SELF(TTableViewDialog);
    if (iContext->argumentCount() == 1)
    {
        cSelf->setSelectionMode(static_cast<QAbstractItemView::SelectionMode>
                                (iContext->argument(0).toInt32()));
    }
    else
    {
        return TEcmaHelper::throwError_FuncDisMatch("TTableViewDialog", "setSelectionMode","Invalid ArgumentCount",iContext);
    }
    return iContext->engine()->undefinedValue();
}
